
--
--  TjDragIt.lua
--    by Tuhljin
--
--  A quick way to make given frames draggable by the player.
--
--  TjDragIt.EnableDragging( myFrame[, child1[, child2[, ... childN]]] )
--    myFrame
--      The frame to be made draggable.
--    child1, child2, ... childN
--      A number of other frames that, when the player tries to drag them, should cause myFrame to move.
--
--  TjDragIt.EnableIndirectDragging( myFrame[, child1[, child2[, ... childN]]] )
--    As TjDragIt.EnableDragging except that it does not set myFrame to be dragged directly, instead relying on
--    the specified child frames to move it.
--
--  If you don't want parts of myFrame to be able to move off the screen, use myFrame:SetClampedToScreen(true).
--


local THIS_VERSION = 0.1

if (not TjDragIt or TjDragIt.Version < THIS_VERSION) then
  TjDragIt = {};
  TjDragIt.Version = THIS_VERSION;

  local parents = {}

  local function MouseDown(frame, button)
    local name = frame:GetName()
    frame = name and parents[name] or frame.TjDragIt_parent or frame
    if (frame:IsMovable() and button == "LeftButton") then
      frame:StartMoving()
    end
  end

  local function MouseUp(frame, button)
    local name = frame:GetName()
    frame = name and parents[name] or frame.TjDragIt_parent or frame
    if (button == "LeftButton") then
      frame:StopMovingOrSizing()
    end
  end

  local function sethook(frame, script, handler)
    local prev = frame:GetScript(script)
    if (prev) then
      frame:HookScript(script, handler)
    else
      frame:SetScript(script, handler)
    end
  end

  function TjDragIt.EnableIndirectDragging(frame, ...)
    frame:SetMovable(true)
    local child, name
    for i=1, select("#", ...) do
      child = select(i, ...)
      child:SetMovable(true)
      child:SetClampedToScreen(false)
      child:EnableMouse(true)
      if (not child.TjDragIt_hooked) then
        sethook(child, "OnMouseDown", MouseDown)
        sethook(child, "OnMouseUp", MouseUp)
        child.TjDragIt_hooked = true
      end
      name = child:GetName()
      if (name) then
        parents[name] = frame
      else
        child.TjDragIt_parent = frame
      end
    end
  end

  function TjDragIt.EnableDragging(frame, ...)
    TjDragIt.EnableIndirectDragging(frame, ...)
    frame:EnableMouse(true)
    if (not frame.TjDragIt_hooked) then
      sethook(frame, "OnMouseDown", MouseDown)
      sethook(frame, "OnMouseUp", MouseUp)
      frame.TjDragIt_hooked = true
    end
  end

  function TjDragIt.DisableDragging(frame, ...)
    frame:SetMovable(false)
    local child, name
    for i=1, select("#", ...) do
      child = select(i, ...)
      child:SetMovable(false)
    end
  end

end
